/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * NotMoleculeByNameAction.cpp
 *
 *  Created on: May 12, 2010
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//#include "CodePatterns/MemDebug.hpp"

#include "molecule.hpp"
#include "CodePatterns/Log.hpp"
#include "CodePatterns/Verbose.hpp"
#include "Descriptors/MoleculeNameDescriptor.hpp"
#include "Descriptors/MoleculeSelectionDescriptor.hpp"
#include "World.hpp"

#include <boost/foreach.hpp>
#include <iostream>
#include <string>

#include "NotMoleculeByNameAction.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "NotMoleculeByNameAction.def"
#include "Action_impl_pre.hpp"
/** =========== define the function ====================== */
ActionState::ptr SelectionNotMoleculeByNameAction::performCall() {
  LOG(1, "Unselecting all molecule called " << params.molname.get());
  std::vector<const molecule *> unselectedMolecules =
      const_cast<const World &>(World::getInstance()).
        getAllMolecules((!MoleculesBySelection()) && MoleculeByName(params.molname.get()));
  World::getInstance().unselectAllMolecules(MoleculeByName(params.molname.get()));
  LOG(0, World::getInstance().countSelectedMolecules() << " molecules remain selected.");

  return ActionState::ptr(new SelectionNotMoleculeByNameState(unselectedMolecules,params));
}

ActionState::ptr SelectionNotMoleculeByNameAction::performUndo(ActionState::ptr _state) {
  SelectionNotMoleculeByNameState *state = assert_cast<SelectionNotMoleculeByNameState*>(_state.get());

  World::getInstance().selectAllMolecules(MoleculeByName(state->params.molname.get()));
  BOOST_FOREACH( const molecule *mol, state->unselectedMolecules)
    World::getInstance().unselectMolecule(mol);

  return ActionState::ptr(_state);
}

ActionState::ptr SelectionNotMoleculeByNameAction::performRedo(ActionState::ptr _state){
  SelectionNotMoleculeByNameState *state = assert_cast<SelectionNotMoleculeByNameState*>(_state.get());

  World::getInstance().unselectAllMolecules(MoleculeByName(state->params.molname.get()));

  return ActionState::ptr(_state);
}

bool SelectionNotMoleculeByNameAction::canUndo() {
  return true;
}

bool SelectionNotMoleculeByNameAction::shouldUndo() {
  return true;
}
/** =========== end of function ====================== */
